// Version 1.0  Jan 20, 2017: Original source code
// Version 2.0  Jan 31, 2017 Changes:
// * Defined fr1...fr3 as member variables of class Field.
// * Optimized sep()
// * Implmented particle density code: pc in Field class, paricle density, min/max for fr1..fr3 as well as  deltaFr? factors
// * Revise & adapt code for atrraction(), update() for all classes, swarm()
// * Revise interaction between fields
// Version 4.0  Feb 06, 2017 Changes:
// * Number of particles instead of density
// * Fixing a cpl of bugs: fr2 update, init of max/min frX values
// * Update of particle threshold via mouseX

import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class particle_fields2 extends PApplet {

  World world;

  boolean running = true;
  boolean showMenue = false;
  boolean showNew = false;
  Slider[] sliders = new Slider[15];
  int particleCount = 2000;
  int fieldCount = 20;
  int seed = 12345;

  Slider[] newSliders = new Slider[3];

  public void setup() {

    world = new World( 20, 2000 );
    background(0);
    colorMode(HSB);

    sliders[0] = new Slider( "d1 Ve", 20, 20, 120, 15, 0, 1, world.f1);
    //COMENT### Next shouldn't be "d2 Ac" instead of "d2 Ve"
    sliders[1] = new Slider( "d2 Ve", 20, 40, 120, 15, 0, 1, world.f2);
    sliders[2] = new Slider( "r1 Se", 20, 60, 120, 15, 0, 200, world.r1);
    sliders[3] = new Slider( "r2 Ali", 20, 80, 120, 15, 0, 200, world.r2);
    sliders[4] = new Slider( "r3 Co", 20, 100, 120, 15, 0, 200, world.r3);
    sliders[5] = new Slider( "p1 Se", 20, 120, 120, 15, 0, .1f, world.pfs);
    sliders[6] = new Slider( "p2 Ali", 20, 140, 120, 15, 0, .01f, world.pfa);
    sliders[7] = new Slider( "p3 Co", 20, 160, 120, 15, 0, 1, world.pfc);
    sliders[8] = new Slider( "d1 Ve", 200, 20, 120, 15, 0, 1, world.ff1);
    sliders[9] = new Slider( "d2 Ac", 200, 40, 120, 15, 0, 1, world.ff2);
    sliders[10] = new Slider( "r1 Se", 200, 60, 120, 15, 0, 300, world.fr1);
    sliders[11] = new Slider( "r2 Ch", 200, 80, 120, 15, 0, 300, world.fr2);
    sliders[12] = new Slider( "r3 Re", 200, 100, 120, 15, 0, 300, world.fr3);
    sliders[13] = new Slider( "p1 Se", 200, 120, 120, 15, 0, 1, world.ffs);
    sliders[14] = new Slider( "p2 Ch", 200, 140, 120, 15, 0, .1f, world.fchase);

    newSliders[0] = new Slider( "seed", 20, 20, 120, 15, 0, 65535, seed);
    newSliders[1] = new Slider( "particles", 20, 40, 120, 15, 0, 5000, particleCount);
    newSliders[2] = new Slider( "fields", 20, 60, 120, 15, 0, 100, fieldCount);
  }

  public void draw() {
    //  background(255);

    noStroke();
    fill(0, 20);
    rect(0, 0, width, height);  
    world.update();  
    world.draw();
    stroke(0);
    /*
  fill(255);
     rect(0,0,200,30);
     fill(0);
     text(""+frameRate,10,20);
     */
    if (showMenue) {
      fill(255, 4);
      //rect(10, 10, 410,200);
      pushMatrix();
      translate(10, 10);
      for ( int i=0; i<sliders.length; i++) {
        sliders[i].draw();
      }
      popMatrix();
    } else if ( showNew ) {
      fill(255, 4);
      //rect(10, 10, 410,200);
      pushMatrix();
      translate(10, 10);
      for ( int i=0; i<newSliders.length; i++) {
        newSliders[i].draw();
      }
      popMatrix();
    }
  }

  public void mousePressed() {
    float mx = mouseX - 10;
    float my = mouseY - 10;
    if (showMenue) {
      for ( int i=0; i<sliders.length; i++) {
        if (sliders[i].inside(mx, my)) {
          sliders[i].active = true;
        }
      }
    } else if (showNew) {
      for ( int i=0; i<newSliders.length; i++) {
        if (newSliders[i].inside(mx, my)) {
          newSliders[i].active = true;
        }
      }
    }
  }

  public void mouseDragged() {
    float px = pmouseX - 10;
    float mx = mouseX - 10;

    if (showMenue) {

      for ( int i=0; i<sliders.length; i++) {
        if (sliders[i].active) {
          sliders[i].slide(px, mx);
          changeParam(i, sliders[i].value);
        }
      }
    } else if (showNew) {
      for ( int i=0; i<newSliders.length; i++) {
        if (newSliders[i].active) {
          newSliders[i].slide(px, mx);
          changeParamNew(i, newSliders[i].value);
        }
      }
    }
  }

  public void mouseReleased() {
    if (showMenue) {
      for ( int i=0; i<sliders.length; i++) {
        sliders[i].active = false;
      }
    } else if ( showNew ) {
      for ( int i=0; i<newSliders.length; i++) {
        newSliders[i].active = false;
      }
      initWorld();
      showNew = false;
    }
  }

  public void initWorld() {
    randomSeed( seed );
    world = new World( fieldCount, particleCount);
    for ( int i=0; i<sliders.length; i++) {
      changeParam( i, sliders[i].value);
    }
    running = true;
  }

  public void changeParam(int idx, float value ) {
    switch(idx) {
    case 0: 
      world.f1 = value; 
      break;
    case 1: 
      world.f2 = value; 
      break;
    case 2: 
      world.r1 = value; 
      break;
    case 3: 
      world.r2 = value; 
      break;
    case 4: 
      world.r3 = value; 
      break;
    case 5: 
      world.pfs = value; 
      break;
    case 6: 
      world.pfa = value; 
      break;
    case 7: 
      world.pfc = value; 
      break;
    case 8: 
      world.ff1 = value; 
      break;
    case 9: 
      world.ff2 = value; 
      break;
    case 10: 
      world.fr1 = value; 
      break;
    case 11: 
      world.fr2 = value; 
      break;
    case 12: 
      world.fr3 = value; 
      break;
    case 13: 
      world.ffs = value; 
      break;
    case 14: 
      world.fchase = value; 
      break;
    }
  }

  public void changeParamNew(int idx, float value ) {
    switch(idx) {
    case 0: 
      seed= PApplet.parseInt(value); 
      break;
    case 1: 
      particleCount = PApplet.parseInt(value); 
      break;
    case 2: 
      fieldCount = PApplet.parseInt(value); 
      break;
    }
  }

  public void saveParams(File file) {
    if ( file  == null ) {
      println( "save cancelled" );
    } else {
      String[] p = new String[18];
      p[0] = ""+seed;
      p[1] = ""+particleCount;
      p[2] = ""+fieldCount;

      for ( int i=0; i<sliders.length; i++) {
        p[i+3] = ""+sliders[i].value;
      }
      saveStrings(file.getAbsolutePath(), p);
    }
    running = true;
  }

  public void loadParams(File file) {
    if ( file  == null ) {
      println( "load cancelled" );
    } else {
      running = false;
      String[] p = loadStrings(file.getAbsolutePath());
      if (p.length == 18) {
        seed = PApplet.parseInt(p[0]);
        particleCount = PApplet.parseInt(p[1]);
        fieldCount = PApplet.parseInt(p[2]);
        for ( int i=0; i<sliders.length; i++) {
          sliders[i].value = PApplet.parseFloat(p[i+3]);
        } 
        initWorld();
      }
      running = true;
    }
  }

  public void keyPressed() {
    //showMenue = false;
    //showNew = false;

    if (key == 'm') {
      showMenue =  !showMenue;
      if (showMenue) showNew = false;
    } else   if (key == 'n') {
      running = false;
      showNew = !showNew;
      if (showNew) showMenue = false;
    } else if ( key == 's' ) {
      running = false;
      selectOutput("save parameters:", "saveParams");
    } else if ( key == 'l' ) {
      running = false;
      selectOutput("load parameters:", "loadParams");
    } else {
      running =! running;  
      if ( running ) { 
        loop();
      } else {
        noLoop();
      }
    }
  }
  class Field {
    float px=0, py=0; 
    float vx=0, vy=0;
    float ax=0, ay=0;
    float fr1;// = 80;            //MUTABLE
    float fr2;// = 100;           //MUTABLE
    float fr3;// = 40;            //MUTABLE

    float min_fr1,min_fr2,min_fr3;            //MUTABLE:
    float max_fr1,max_fr2,max_fr3;            //MUTABLE: 
    float deltafr1, deltafr2, deltafr3;       //MUTABLE

    int pc=0;

    public void update() {  
      float tx=px;
      float ty=py;

      vx*=world.ff1;
      vy*=world.ff1;
      vx+=ax;
      vy+=ay;
      ax*=world.ff2;
      ay*=world.ff2;
      px+=vx;
      py+=vy;

      if (world.open) {
        if ( px < 0 ) px+= width;
        if ( py < 0 ) py+= height;
        if (px > width) px-= width;
        if (py > height)  py -=height;
      } else {
        if (px > width || px < 0) {
          vx = -vx;
          ax = -ax;
          px = tx;
          py = ty;
        }
        if (py > height || py < 0) {
          vy = -vy;
          ay = -ay;
          px = tx;
          py = ty;
        }
      }
    }

    public void addForce( float fx, float fy ) {
      ax+=fx;
      ay+=fy;
    }
      
    public void setFrFieldLimits(){
	min_fr1=0.50*fr1; min_fr2=0.50*fr2; min_fr3=0.50*fr3;   //MUTABLE: only factors
	max_fr1=1.20*fr1; max_fr2=1.20*fr2; max_fr3=1.20*fr3;   //MUTABLE: only factors
	deltafr1=1; deltafr2=1; deltafr3=1;                     //MUTABLE
    }

    public void draw() {
      stroke(0, 255, 255);
      fill(0, 255, 255);
      ellipse(px, py, 10, 10);
      noFill();
      stroke(0, 255, 128, 30);
      ellipse(px, py, this.fr1, this.fr1);
      stroke(0, 255, 128, 10);
      ellipse(px, py, this.fr3*1.5f, this.fr3*1.5f);
      stroke(20, 255, 128, 10);
      for ( int i=0; i<world.fields.length; i++) {
        Field other = world.fields[i];
        if ( other != this ) {
          if ( dist(px, py, other.px, other.py) < 150 ) {
            line(px, py, other.px, other.py);
          }
        }
      }
    }
  }

    
  class Particle {
    float px=0, py=0; 
    float vx=0, vy=0;
    float ax=0, ay=0;

    World world;

    Particle( World world ) {
      this.world = world;
    }

    public void update() {
      float tx=px;
      float ty=py;

      vx*=world.f1;
      vy*=world.f1;
      vx+=ax;
      vy+=ay;
      ax*=world.f2;
      ay*=world.f2;
      px+=vx;
      py+=vy;

      if (world.open) {
        if ( px < 0 ) px+= width;
        if ( py < 0 ) py+= height;
        if (px > width) px-= width;
        if (py > height)  py -=height;
      } else {
        if (px > width || px < 0) {
          vx = -vx;
          ax = -ax;
          px = tx;
          py = ty;
        }
        if (py > height || py < 0) {
          vy = -vy;
          ay = -ay;
          px = tx;
          py = ty;
        }
      }
    }

    public void addForce( float fx, float fy ) {
      ax += fx;
      ay += fy;
    }

    public void draw() {
      float vel = sqrt(sq(vx)+sq(vy))*10;    
      stroke(255-min(vel, 10)*25.5f, 255, 255);
      rect( px, py, 0, 0);
      //point(px,py);  //HERE
    }
  }
  class Slider {
    float x, y, w, h;
    float value;
    float min, max;
    String name;

    boolean active = false;

    Slider( String name, float x, float y, float w, float h, float min, float max, float value ) {
      this.name = name;
      this.x = x;
      this.y = y;
      this.w = w;
      this.h = h;
      this.min = min;
      this.max = max;
      this.value = value;
    }

    public void draw() {
      fill(200);
      text(name, x, y+h-3 );
      stroke(50);
      fill(50);
      rect(x+50, y, w, h);
      noStroke();
      fill(active?200:128);
      rect(x+50, y, (value-min)/(max-min)*w, h+1);
      fill(250);
      text(""+value, x+60, y+h-3 );
    }

    public boolean inside(float tx, float ty) {
      return (tx >= 50+x && tx <= x+50+w && ty>=y && ty <= y+h);
    }

    public void slide(float px, float mx) {
      value+=(mx-px)*(max-min)/w+min;
      if ( value < min ) value = min;
      if ( value > max ) value = max;
    }
  }
  class World {
    // damping factor for velocity
    float f1 = .98f;
    // damping factor for forces
    float f2 = .3f;

    float r1 = 4;
    float r2 = 30;
    float r3 = 60;

    float pfs = 0.05f;
    float pfa = 0.005f;
    float pfc = .4f;  

    // damping factor for velocity
    float ff1 = .80f;
    // damping factor for forces
    float ff2 = .3f;

    float ffs = 0.1f;
    float fchase = 0.01f;
    float fr1 = 80;//80;
    float fr2 = 80;//100;
    float fr3 = 40;//40;
      
    float parnThresh=10;            //MUTABLE
      //float densityThresh=0.001;  //MUTABLE
      
    boolean open=false;

    private Particle[] particles;
    private Field[] fields;

    World( int fieldcount, int particlecount) {
      fields = new Field[fieldcount];
      particles = new Particle[particlecount];
      for ( int i=0; i<fields.length; i++) {
        fields[i] = new Field();
        fields[i].px = random(width*.8f+50);
        fields[i].py = random(height*.8f+50);
        fields[i].fr1=fr1;//random(fr1*0.50, fr1*1.1);
        fields[i].fr2=fr2;//random(fr2*0.50, fr2*1.1);
        fields[i].fr3=fr3;//random(fr3*0.50, fr3*1.1);
	fields[i].setFrFieldLimits();
      }
      
      for ( int i=0; i< particles.length; i++) {
        particles[i] = new Particle(this);
        particles[i].px = random(width);
        particles[i].py = random(height);
      }
    }

    public void update() {

      //if (frameCount%120==0) {
      //  fr3+=dfr3;
      //  if (fr3>50 || fr3<30)
      //    dfr3=-dfr3;
      //}

      for ( int i=0; i< particles.length; i++) {      
        for ( int j=0; j<fields.length; j++) {

          float lx = particles[i].px - fields[j].px;
          float ly = particles[i].py - fields[j].py;

          if (open) {
            if ( abs(lx) > abs(lx+width)) lx+=width;
            if ( abs(ly) > abs(ly+height)) lx+=height;
          }

          float l = wdist(particles[i].px, particles[i].py, fields[j].px, fields[j].py);
          if ( l< fields[j].fr3 && l> 1) {         
            //float f = 10.0/sq(l);
            float f = 1.0f/l;
            lx = lx/l*f;
            ly = ly/l*f;          
            particles[i].addForce(lx, ly);
          }
        }
      }

      //COMENT### Fater way to compare between field pairs and using action-reaction law
      //for (int i=0; i<n-1; i++)
      //for (int j=i+1; j<n; j++) {
      //println("FWD pair="+i+" "+j);
      //println("           BCK pair="+j+" "+i);
      //}
      //sep();
      for ( int i=0; i<fields.length; i++) {
        sep( fields[i] );
        attract( fields[i] );
        fields[i].update();
      }
      for ( int i=0; i< particles.length; i++) {
        swarm(particles[i]);      
        particles[i].update();
      }

      applyDensityCorrections();
    }

    public void applyDensityCorrections() {
      for ( int i=0; i<fields.length; i++) {
        Field f=fields[i];
        //float cpd=f.pc/float(particleCount);     //Current particle density
	float cpn=f.pc;                            //Current particle number

        if (cpn<parnThresh) {

          if (f.fr1>f.min_fr1) f.fr1-=f.deltafr1;
          if (f.fr2>f.min_fr2) f.fr2-=f.deltafr2;  //ERROR
          if (f.fr3>f.min_fr3) f.fr3-=f.deltafr3;
        } else {

          if (f.fr1<f.max_fr1) f.fr1+=f.deltafr1;
          if (f.fr2<f.max_fr2) f.fr2+=f.deltafr2;
          if (f.fr3<f.max_fr3) f.fr3+=f.deltafr3;
        }
      }
    }

    public void draw() {
      //Only if mouse position changes
      if(mouseX!=pmouseX){
	parnThresh=map(mouseX,0,width,5,150);
	println("Field threshold updated to "+parnThresh);
      }
	
      for ( int i=0; i<fields.length; i++) {
        fields[i].draw();
      }
      for ( int i=0; i< particles.length; i++) {
        //COMMENT###
        particles[i].draw();
      }
    }

    public void swarm(Particle p) {
      float tsx=0, tsy=0;        
      float tax=0, tay=0;
      float tcx=0, tcy=0;

      int counts = 0;
      int counta = 0;
      int countc = 0;
      for (int i=0; i<world.particles.length; i++) {
        Particle other = particles[i];
        if ( p != other) {

          float d = wdist(p.px, p.py, other.px, other.py);
          //float d = 100;


          if ( d < r1) {
            counts++;

            tsx+=other.px;
            tsy+=other.py;
          } else if (d < r3) {        
            countc++;            
            tcx+=other.px;
            tcy+=other.py;
          }  
          if ( d < r2) {
            counta++;
            tax+=other.vx;
            tay+=other.vy;
          }
        }
      }
      if ( counts > 0 ) { 
        tsx/=counts;
        tsy/=counts;
        tsx = p.px - tsx;
        tsy = p.py - tsy;

        float l = wdist(0, 0, tsx, tsy);
        if ( l > 0) {
          tsx/=l;
          tsy/=l;
        }
        tsx*=pfs;
        tsy*=pfs;
        p.addForce(tsx, tsy);
      }

      if ( counta > 0 ) {
        tax = tax/counta;
        tay = tay/counta;

        float l = wdist(tax, tay, 0, 0);
        if (l > 0) {
          tax=tax/l;
          tay=tay/l;
        }
        tax*=pfa;
        tay*=pfa;
        p.addForce(tax, tay);
      }

      if ( countc> 0 ) {
        tcx/=countc;
        tcy/=countc;
        tcx = p.px - tcx;
        tcy = p.py - tcy;

        float l = wdist(0, 0, tcx, tcy);
        if (l > 0) {
          tcx/=l;
          tcy/=l;
        }
        tcx*=(pfc-0.5f)/1000.0f;
        tcy*=(pfc-0.5f)/1000.0f;
        p.addForce(tcx, tcy);
      }
    }

    public void sep() {
      for (int i=0; i<fields.length-1; i++) {
        Field f=fields[i];
        for (int j=i+1; j<fields.length; j++) {
          float tmpx =0, tmpy =0;
          //int count = 0;
          Field other=fields[j];
          float d = wdist(f.px, f.py, other.px, other.py);
          tmpx=f.px - other.px;
          tmpy=f.py - other.py;
          float l = dist(0, 0, tmpx, tmpy);  //COMMENT### dist() for wdit()
          if (l!=0) {
            tmpx/=l;
            tmpy/=l;
          }
          if ( d < f.fr1) {		 
            f.addForce(tmpx*ffs, tmpy*ffs);
          }
          if ( d < other.fr1) {
            other.addForce(-tmpx*ffs, -tmpy*ffs);  //INVERT signs
          }
        }
      }
    }


    public void sep(Field f) {
      float tmpx =0, tmpy =0;
      int count = 0;
      for (int i=0; i<fields.length; i++) {
        Field other = fields[i];
        if ( f != other) {
          float d = wdist(f.px, f.py, other.px, other.py); 
          if ( d < fr1) {
            count++;
            tmpx+=other.px;
            tmpy+=other.py;
          }
        }
      }
      if ( count > 0 ) {      
        tmpx/=count;
        tmpy/=count;
        tmpx = f.px - tmpx;
        tmpy = f.py - tmpy;

        float l = wdist(0, 0, tmpx, tmpy);
        tmpx/=l;
        tmpy/=l;
        tmpx*=ffs;
        tmpy*=ffs;
        f.addForce(tmpx, tmpy);
      }
    }

    public void attract(Field f) {
      float tmpx =0, tmpy =0;
      int count = 0;
      f.pc=0;
      for (int i=0; i<particles.length; i++) {
        Particle other = particles[i];

        float d = wdist( f.px, f.py, other.px, other.py); 
        if ( d < f.fr2) {
          count++;
          tmpx+=other.px;
          tmpy+=other.py;
        }

        if ( d < f.fr1 )  //COMMENT###
          f.pc++;
      }

      if ( count > 0 ) {    
        tmpx/=count;
        tmpy/=count;
        tmpx = tmpx - f.px;
        tmpy = tmpy - f.py;
        float l = wdist(0, 0, tmpx, tmpy);
        tmpx/=l;
        tmpy/=l;
        tmpx*=fchase;
        tmpy*=fchase;

        f.addForce(tmpx, tmpy);
      }
    }

    public float wdist( float x1, float y1, float x2, float y2 ) {
      if ( !open ) {
        return dist( x1, y1, x2, y2);
      } else { 
        float dx = min( abs(x1-x2), abs(x1+width-x2));
        float dy = min( abs(y1-y2), abs(y1+height-y2));
        //COMENT### Checking min independently of x and y is not sufficient.
        //If distance is closer along one dimension when warping around,
        //it could not be close in the other. It needs to be checked!
        return sqrt( sq(dx) + sq(dy));
      }
    }
  }
  public void settings() {  
    size(600, 400);
  }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "particle_fields2" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
