//REFERENCE: https://forum.processing.org/two/discussion/20350/particle-detection-within-a-field#latest
//REFERENCE: https://forum.processing.org/two/discussion/18745/tensors
//REFERENCE: https://www.openprocessing.org/sketch/6886#
//REFERENCE: https://github.com/craftoid/sketches/blob/master/bitcraft/tapeworm/bubble.pde

//COMMAND:::  processing-java --sketch=%cd%\particle_fields2 --output=%cd%\particle_fields2\sk\out --force --run

// Version 1.0  Jan 20,2017: Original source code
// Version 2.0  Jan 31, 2017 Changes:
// * Defined fr1...fr3 as member variables of class Field.
// * Optimized sep()
// * Implmented particle density code: pc in Field class, paricle density, min/max for fr1..fr3 as well as  deltaFr? factors
// * Revise & adapt code for atrraction(), update() for all classes, swarm()
// * Revise interaction between fields
// Version 4.0  Feb 06, 2017 Changes:
// * Number of particles instead of density
// * Fixing a cpl of bugs: fr2 update, init of max/min frX values
// * Update of particle threshold via mouseX
// Version 5.0  Feb 12, 2017 Changes:
// * invertThreshEffect: Invert effect on fields. To grow when density is below thresh instead
// * densityFlag: To adjust theshold density only when status to do so is enabled. Threshold is adjust using mouseX.
// *The following line was change  in class Slide, function slide(float px, float mx):   value+=(mx-px)*(max-min)/w;//+min    in order to manage negative values.
// * Change the way the running boolean variable(flag) affects the simulation (updateRunningStatus)
// * selectInput used when loading a file session (instead of selectOutput)
// *
// Version 6.0  Feb 26, 2017 Changes:
// * Changes of calling init() when either menu is switch off
// * NEW: updateAllFields() to update the values of each field when it has changed.
// * Set p2 to zero to disable attraction between fields
// * Changes in the menu now directly updates the respective fields
// * Implementation of sep_tapeworm() as an alternative to sep(). The concept of this function is to push the fields away when they overlapped, as dictated by the valuer of fr1 involved in the instant interaction.
// * Fix fr3 to a value of 10. Its delta is <<1 which it was settle in previous emails.
// * Implementation of verbosity to inquire for system variables: printInformation() function and key 'v' to toggle it.
// *
// Version 7.0  Mar 10, 2017 Changes:
// * Changes between initWorld, World constructor, changePar and changeParNew so the world is not recreated but its content is just updated.
// * Introduction of dfr1Flag to keep track when the radius of a field when changing. used in collisionDetection
// * fr1/2/3 renamed master_fr1/2/3
// * Implementing collision of pair interactions as additive force in post-step
// *
// Version 8.0  Mar 19, 2017 Changes:
// * Implementation of help menu display ('h'), remove visual lines btw fields ('c')
// * Loading/saving session takes into account parNThresh and invertThreshEffect
// * Definition of collide_tapeWorms() within World class and collide_tapeWorms(Field) within Field class: Forces Field not to overlap!
// * Changes in vervosity from boolean to int to handle multiple verbosity levels.
// * Disable loop/noloop calls and instead, running field manages if sketch gets updated. This ensures slides are active and avoid bugs when 
// *     recreating field and particle objects via menu.
// * Active update of slides from menu (key 'm') reflected in sketch
// * Implementation of random field positioning at intiWorld() ensuring not overlap in the process
// * Change of field update and field changes during init world. Maxt/min fields are defined from 0.5*fr# to 1.0*fr#
// * implementation of algorithm to flip velocity of fields when bouncing in wall. Prototype function that works only for the
// *     collide_tapeWorm() function. Notice this function controls wall collisions and take into account field sizes.
// *     Wall colision algorithm under Field::Update() is disabilited by previous function as code doesn't take into account field size
// *     and therefore, they never get executed.
// * TBD: Forces managment in the function has been disabled to avoid overun speeds and out of control runs.
// *      Any further implmentation requires heavy revising of code structure as it is becoming very heavy to maintain.
// *
// Version 9.0  Mar 26, 2017 Changes:
// * Remove of the key 'd' so to adjust parNThresh via mouseX couple to width dimension of the screen
// * The key 'c' beside removing interconnecting lines, adjust background alpha effect and alpha stroke in Field's objects
// * Interconnecting lines is re-defined as fr1... No specific reason but for observations
// * New slider menu has two new slides: width and height which control sketch size
// * Definition of World::minFR1 which defines min field size for fr1/2/3 from a range of 5 to 100% of their nominal value. Control via slider
// * parNThreshold is now control via slider
// * Starting Field speed is defined using a Slider coupled to World::fieldSpeed
// * Definition of key 'r' for recording frames into the Output\\ folder using saveFrame command
// * Update showHelp menu
// * Following functions were updated or changed: changeParam(), changeParamNew(), updateAllFields(), init/re-init field's velocity (speed)
// * 


import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class particle_fields4 extends PApplet {

    World world;

    final int kMaxVerbosity=3;  // 0: no verbose  1=all state vars  2=collisionCheck output
    final int kMaxSliders = 18;
    final int kMaxNewSliders=5;
    
    int verbosity=0;
    boolean showFieldLines=false;
    boolean recording = false;

    boolean running = true;
    boolean showMenue = false;
    boolean showNew = false;
    boolean densityFlag=false;    
    Slider[] sliders = new Slider[kMaxSliders];
    
    int seed = 12345;
    int fieldCount = 32;
    int particleCount = 2000;
    int wWidth=600;
    int wHeight=400;

    String vers="v26C17";

    Slider[] newSliders = new Slider[kMaxNewSliders];


    // ####################################################################################################
    //
    // ####################################################################################################
    public void settings() {  
	size(wWidth,wHeight);
	//size(600, 100);
    }


    // ####################################################################################################
    //
    // #################################################################################################### ??.   
    public void setup() {

	world = new World( fieldCount, particleCount );    
	background(0);
	colorMode(HSB);
	surface.setTitle("Version " + vers);
	surface.setResizable(true);
	ellipseMode(RADIUS);

	sliders[0] = new Slider( "d1 Ve", 20, 20, 120, 15, 0, 1, world.f1);	
	sliders[1] = new Slider( "d2 Ve", 20, 40, 120, 15, 0, 1, world.f2);  //COMENT### This should be "d2 Ac" instead of "d2 Ve"
	sliders[2] = new Slider( "r1 Se", 20, 60, 120, 15, 0, 200, world.r1);
	sliders[3] = new Slider( "r2 Ali", 20, 80, 120, 15, 0, 200, world.r2);
	sliders[4] = new Slider( "r3 Co", 20, 100, 120, 15, 0, 200, world.r3);
	sliders[5] = new Slider( "p1 Se", 20, 120, 120, 15, 0, .1f, world.pfs);
	sliders[6] = new Slider( "p2 Ali", 20, 140, 120, 15, 0, .01f, world.pfa);
	sliders[7] = new Slider( "p3 Co", 20, 160, 120, 15, 0, 1, world.pfc);
	
	sliders[8] = new Slider( "d1 Ve", 200, 20, 120, 15, -1, 1, world.ff1);
	sliders[9] = new Slider( "d2 Ac", 200, 40, 120, 15, -1, 1, world.ff2);
	sliders[10] = new Slider( "r1 Se", 200, 60, 120, 15, 0, 300, world.master_fr1);
	sliders[11] = new Slider( "r2 Ch", 200, 80, 120, 15, 0, 300, world.master_fr2);
	sliders[12] = new Slider( "r3 Re", 200, 100, 120, 15, 0, 300, world.master_fr3);
	sliders[13] = new Slider( "p1 Se", 200, 120, 120, 15, 0, 1, world.ffs);
	sliders[14] = new Slider( "p2 Ch", 200, 140, 120, 15, 0, .1f, world.fchase);

	sliders[15] = new Slider( "parNThres", 200, 180, 120, 15, 1, 150, world.parNThresh);
	sliders[16] = new Slider( "%minFR1",   200, 200, 120, 15, 0.05, 1.0f, world.minFR1);
	sliders[17] = new Slider( "Speed",     200, 220, 120, 15, 0, 15.0f, world.fieldSpeed);
	

	newSliders[0] = new Slider( "seed     ", 20, 20, 120, 15, 0, 65535, seed);
	newSliders[1] = new Slider( "particles", 20, 40, 120, 15, 0, 5000, particleCount);
	newSliders[2] = new Slider( "fields   ", 20, 60, 120, 15, 0, 100, fieldCount);
	newSliders[3] = new Slider( "width    ", 20, 80, 120, 15, 100, 1080, wWidth);
	newSliders[4] = new Slider( "height   ", 20,100, 120, 15, 100, 1080, wHeight);

	//world.initWorld(sliders);
    }

    // ####################################################################################################
    //
    // ####################################################################################################    
    public void draw() {
	
	
	if(showFieldLines==true){
	    noStroke();
	    fill(0, 20);
	    rect(0, 0, width, height);
	}
	else
	    background(0);
	
	if(running)
	    world.update();  
	world.draw();
	stroke(0);

	if (recording) 
           saveFrame("/output/particle_fields4_#####.png");
  
	
	/*
	  fill(255);
	  rect(0,0,200,30);
	  fill(0);
	  text(""+frameRate,10,20);
	*/
	if (showMenue) {
	    fill(255, 4);
	    //rect(10, 10, 410,200);
	    pushMatrix();
	    translate(10, 10);
	    for ( int i=0; i<sliders.length; i++) {
		sliders[i].draw();
	    }
	    popMatrix();
	} else if ( showNew ) {
	    fill(255, 4);
	    //rect(10, 10, 410,200);
	    pushMatrix();
	    translate(10, 10);
	    for ( int i=0; i<newSliders.length; i++) {
		newSliders[i].draw();
	    }
	    popMatrix();
	}
    }

    // ####################################################################################################
    //
    // ####################################################################################################    
    public void mousePressed() {
	float mx = mouseX - 10;
	float my = mouseY - 10;
	if (showMenue) {
	    for ( int i=0; i<sliders.length; i++) {
		if (sliders[i].inside(mx, my)) {
		    sliders[i].active = true;
		}
	    }
	} else if (showNew) {
	    for ( int i=0; i<newSliders.length; i++) {
		if (newSliders[i].inside(mx, my)) {
		    newSliders[i].active = true;
		}
	    }
	}
    }

    // ####################################################################################################
    //
    // ####################################################################################################    
    public void mouseDragged() {
	float px = pmouseX - 10;
	float mx = mouseX - 10;

	if (showMenue) {

	    for ( int i=0; i<sliders.length; i++) {
		if (sliders[i].active) {
		    sliders[i].slide(px, mx);
		    world.changeParam(i, sliders[i].value);
		}
	    }
	} else if (showNew) {
	    for ( int i=0; i<newSliders.length; i++) {
		if (newSliders[i].active) {
		    newSliders[i].slide(px, mx);
		    world.changeParamNew(i, newSliders[i].value);
		}
	    }
	}
    }

    

    // ####################################################################################################
    //
    // ####################################################################################################    
    public void mouseReleased() {
	if (showMenue) {
	    for ( int i=0; i<sliders.length; i++) {
		sliders[i].active = false;
	    }
	} else if ( showNew ) {
	    for ( int i=0; i<newSliders.length; i++) {
		newSliders[i].active = false;
	    }
	    //initWorld();          //CHANGED: particle_fields4
	    //showNew = false;      //CHANGED: particle_fields4
	}
    }



    // ####################################################################################################
    //
    // ####################################################################################################    
    public void saveParams(File file) {
	if ( file  == null ) {
	    println( "save cancelled" );
	} else {
	    String[] p = new String[20];
	    p[0] = ""+seed;
	    p[1] = ""+particleCount;
	    p[2] = ""+fieldCount;

	    for ( int i=0; i<sliders.length; i++) {
		p[i+3] = ""+sliders[i].value;
	    }
	    p[18]=""+world.parNThresh;
	    p[19]=""+world.invertThreshEffect;
	    saveStrings(file.getAbsolutePath(), p);
	}
	updateRunningStatus(true);
    }

    // ####################################################################################################
    //
    // ####################################################################################################       
    public void loadParams(File file) {
	if ( file  == null ) {
	    println( "load cancelled" );
	} else {
	    updateRunningStatus(false);
	    String[] p = loadStrings(file.getAbsolutePath());
	    if (p.length >= 18) {
		seed = PApplet.parseInt(p[0]);
		particleCount = PApplet.parseInt(p[1]);
		fieldCount = PApplet.parseInt(p[2]);
		for ( int i=0; i<sliders.length; i++) {
		    sliders[i].value = PApplet.parseFloat(p[i+3]);
		} 
		world.initWorld(newSliders, sliders);		
	    }

	    if (p.length == 20){
		world.parNThresh=PApplet.parseInt(p[18]);
	        world.invertThreshEffect=PApplet.parseBoolean(p[19]);
	    }
	    else{
		//Handle previous files where these values were not defined.
		world.parNThresh=10;
	        world.invertThreshEffect=false;
	    }
	    updateRunningStatus(true);
	}
    }

    // ####################################################################################################
    //
    // #################################################################################################### ??.       
    public void keyPressed() {
	//showMenue = false;
	//showNew = false;

	if (key == 'm') {
	    showMenue =  !showMenue;
	    if (showMenue) showNew = false;
	} else   if (key == 'n') {
	    updateRunningStatus(false);
	    showNew = !showNew;
	    if (showNew) showMenue = false;
	} else if ( key == 's' ) {
	    updateRunningStatus(false);
	    selectOutput("save parameters:", "saveParams");
	} else if ( key == 'l' ) {
	    updateRunningStatus(false);
	    selectInput("load parameters:", "loadParams");
	} else if ( key == 'i' || key == 'I') {
	    world.invertThreshEffect=!world.invertThreshEffect;
	    println("Invert field flag: "+world.invertThreshEffect);
	} else if ( key == 'q' || key == 'Q') {
	    System.exit(0);
	} else if ( key == 'c' || key == 'C') {
	    showFieldLines=!showFieldLines;
	} else if (key == 'r' || key == 'R') { 
            recording = !recording;
        }else if ( key == 'h' || key == 'H') {
	    showHelp();
	} else if ( key == 'v' ) {
	    verbosity=(verbosity+1)%kMaxVerbosity;
	    surface.setTitle("Version " + vers + "Verbosity="+verbosity);
	    //} else if ( key == 'd' ) {
	    //if (running==true) {
	    //	densityFlag = !densityFlag;
	    //	println("Density flag is now "+densityFlag+". Current value="+world.parNThresh);
	    //} else {
	    //	println("Density flag can only be change in running mode. Nothing done");
	    //}
	} else {
	    updateRunningStatus(!running);
	}

	if( (key =='m' || key=='n') && showMenue==false && showNew==false)
	    //  world.initWorld(sliders);
	    updateRunningStatus(true);
	
	if ( (key=='n') && showMenue==false && showNew==false)
	    world.initWorld(newSliders, sliders);
    }

    // ####################################################################################################
    //
    // ####################################################################################################
    public void showHelp(){
	println("");
	println("\t####################################################################################################");
	println("\t                                          QUICH HELP");
	println("\t####################################################################################################");
	println("\tc  Show/hide interconnecting lines btween field's centers");
	//println("\td   Enable/Disabled active change of n neighboring particle threshold");
	println("\th   Show this help");
	println("\ti   Invert current nParThresh behavior");
	println("\tl   Load previous saved session");
	println("\tm   Show particleand field parameter menu");
	println("\tn   Show world menu");
	println("\tq   Exit program");
	println("\tr   Toggle recording frame to the Output\\ folder");
	println("\ts   Save current session");
	println("\tv   Activates different levels of output:0=none 1=State pars. 2=Collision print out");
	println("");
	println("\tANY other key will pause the simulation");
	println("");
	println("");
    }

    // ####################################################################################################
    //
    // ####################################################################################################    
    public   void updateRunningStatus(boolean b) {
	running=b;
	if ( running ) { 
	    //loop();
	} else {
	    //noLoop();
	}
    }




    // ####################################################################################################
    //
    // ####################################################################################################   ??. 
    class Field {
	float px=0, py=0; 
	float vx=0, vy=0;
	float ax=0, ay=0;
	float fr1;// = 80;            //MUTABLE
	float fr2;// = 100;           //MUTABLE
	float fr3;// = 40;            //MUTABLE

	float min_fr1, min_fr2, min_fr3;            //MUTABLE:
	float max_fr1, max_fr2, max_fr3;            //MUTABLE: 
	float deltafr1, deltafr2, deltafr3;       //MUTABLE

	int pc=0;
	boolean dfr1Flag;
	boolean speedXflipFlag;
	boolean speedYflipFlag;

	ArrayList<Field> colArray;
	ArrayList<Float> forXArray;
	ArrayList<Float> forYArray;

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD
	public Field() {
	    colArray=new ArrayList<Field>();
	    forXArray=new ArrayList<Float>();
	    forYArray=new ArrayList<Float>();
	    vx=0;
	    vy=0;
	    dfr1Flag=false;            //Keeps track when main radius has experienced any size change
	    speedXflipFlag=false;
	    speedYflipFlag=false;
	}
	
	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD
	public void update() {        
	    float tx=px;
	    float ty=py;

	    if (verbosity==1) println(colArray.size() + " Field vx="+nfs(vx,3,3)+" px="+nfs(px,3,3)+" py="+nfs(py,3,3));

	    //DISABLED::
	    //vx*=world.ff1;
	    //vy*=world.ff1;
	    vx+=ax;
	    vy+=ay;
	    //ax*=world.ff2;
	    //ay*=world.ff2;
	    //-------------
	    
	    px+=vx;
	    py+=vy;

	    for(int i=0;i<forXArray.size();i++){
		px+=forXArray.get(i);
		py+=forYArray.get(i);
	    }

	    if (world.open) {
		if ( px < 0 ) px+= width;
		if ( py < 0 ) py+= height;
		if (px > width) px-= width;
		if (py > height)  py -=height;
	    } else {
		if (px > width || px < 0) {
		    vx = -vx;
		    ax = -ax;
		    px = tx;
		    py = ty;
		}
		if (py > height || py < 0) {
		    vy = -vy;
		    ay = -ay;
		    px = tx;
		    py = ty;
		}
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD
	public void addForce( float fx, float fy ) {      
	    ax+=fx;
	    ay+=fy;
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD
	public void updateAllFields(float _fr1, float _fr2, float _fr3, float lowper) {	    
	    fr1=_fr1;//random(fr1*0.50, fr1*1.1);
	    fr2=_fr2;//random(fr2*0.50, fr2*1.1);
	    fr3=_fr3;//random(fr3*0.50, fr3*1.1);
	    setFrFieldLimits(lowper);
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD
	public void setFrFieldLimits(float lowper) {                   

	    min_fr1=lowper*fr1; 
	    min_fr2=lowper*fr2; 
	    min_fr3=lowper*fr3;   //MUTABLE: only factors
	    max_fr1=fr1; 
	    max_fr2=fr2; 
	    max_fr3=fr3;        //MUTABLE: only factors
	    deltafr1=1; 
	    deltafr2=1; 
	    deltafr3=0.0025;    //MUTABLE
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD        
	public void draw() {

	    
	   
	    noFill();
	    stroke(0, 255, 128,showFieldLines==false?255:30);
	    ellipse(px, py, this.fr1, this.fr1);  //BIGGER

	    //stroke(0, 255, 128, 10);
	    //ellipse(px, py, this.fr3*1.0f, this.fr3*1.0f);

	    if(showFieldLines==false)
		return;

	    stroke(0, 255, 255);
	    fill(0, 255, 255);
	    ellipse(px, py, 5, 5);

	    stroke(20, 255, 128, 10);
	    for ( int i=0; i<world.fields.length; i++) {
		Field other = world.fields[i];
		if ( other != this ) {
		    if ( dist(px, py, other.px, other.py) < this.fr1 ) {  //150 
			line(px, py, other.px, other.py);
		    }
		}
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD
	@Override
	public String toString() {                              
	    String str=super.toString();
	    str+="Pos:"+px+" "+py;
	    str+=" Vel:"+vx+" "+vy;
	    return str;
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS FIELD
	public boolean collide_tapeWorms(Field f) {
	    if (f == this) return false;  // no self collision

	    float d = dist(px, py, f.px, f.py);
	    float rr =  fr1 + f.fr1;

	    // correct bubble positions
	    if (d<rr) {
		float t = (rr-d)/rr;
		float dx =  t * (px-f.px);
		float dy =  t * (py-f.py);
		px += dx; 
		py += dy;
		f.px -= dx; 
		f.py -= dy;
		return true;
	    }
	    return false;
	}
	
	// ####################################################################################################
	//
	// ####################################################################################################  ??.
	//CLASS FIELD
	// https://processing.org/examples/circlecollision.html
	// https://forum.processing.org/two/discussion/comment/89968/#Comment_89968
	public void checkCollision(Field ofield) {

	    //WRAPPER to function
	    PVector position =new PVector(px, py);
	    PVector otherPos =new PVector(ofield.px, ofield.py);
	    PVector velocity = new PVector(vx, vy);
	    PVector otherVel =new PVector(ofield.vx, ofield.vy);
	    //float r = fr1;	    

	    // get distances between the balls components
	    PVector bVect = PVector.sub(otherPos, position);

	    // calculate magnitude of the vector separating the balls
	    float bVectMag = bVect.mag();

	    boolean found=false;
	    for (Field cf : colArray)
		if (cf==ofield){ 
		    found=true;
		    break;
		}

	    //Now if found is true, two cases:
	    //1. If overlap is still going, then do not run overlap correction algorithm. Simply return
	    //2. If no overlap anymore, then remove object from list

	    if (bVectMag > (fr1 + ofield.fr1)) {    
		if (found==true) {
		    int ii=colArray.indexOf(ofield);
		    colArray.remove(ii);  //REF: http://docs.oracle.com/javase/1.5.0/docs/api/java/util/ArrayList.html
		    forXArray.remove(ii);
		    forYArray.remove(ii);
		    int jj=ofield.colArray.indexOf(this);
		    ofield.colArray.remove(jj);
		    ofield.forXArray.remove(jj);
		    ofield.forYArray.remove(jj);
		    
		}
	    } else {

		//Force rechecking of dinamics when fr1 has changed for any of the two fields involved
		//if(dfr1Flag==true || ofield.dfr1Flag==true)
		//    found=false;

		if (found==true) {
		    //    println(nf(frameCount,7)+"Overlapping and inside collison array");
		    
		    //int ii=colArray.indexOf(ofield);
		    //px+=forXArray.get(ii);
		    //py+=forYArray.get(ii);
		
		    //int jj=ofield.colArray.indexOf(this);
		    //ofield.px+=ofield.forXArray.get(jj);
		    //ofield.py+=ofield.forYArray.get(jj);
		    
		    return;    //MONEY LINE MONEY LINE MONEY LINE MONEY LINE
		}

		if (verbosity==2){
		    println("***"+nf(frameCount,7));
		    println(colArray.size()       +" Correcting this  f: "+toString() + " -- "+(bVectMag<(fr1 + ofield.fr1)));
		    println(ofield.colArray.size()+" Correcting other f: "+ofield.toString());
		}
		
		
		// get angle of bVect
		float theta  = bVect.heading();
		// precalculate trig values
		float sine = sin(theta);
		float cosine = cos(theta);

		/* bTemp will hold rotated ball positions. You 
		   just need to worry about bTemp[1] position*/
		PVector[] bTemp = {
		    new PVector(), new PVector()
		};

		/* this ball's position is relative to the other
		   so you can use the vector between them (bVect) as the 
		   reference point in the rotation expressions.
		   bTemp[0].position.x and bTemp[0].position.y will initialize
		   automatically to 0.0, which is what you want
		   since b[1] will rotate around b[0] */
		bTemp[1].x  = cosine * bVect.x + sine * bVect.y;
		bTemp[1].y  = cosine * bVect.y - sine * bVect.x;

		// rotate Temporary velocities
		PVector[] vTemp = {
		    new PVector(), new PVector()
		};

		vTemp[0].x  = cosine * velocity.x + sine * velocity.y;
		vTemp[0].y  = cosine * velocity.y - sine * velocity.x;
		vTemp[1].x  = cosine * otherVel.x + sine * otherVel.y;
		vTemp[1].y  = cosine * otherVel.y - sine * otherVel.x;

		/* Now that velocities are rotated, you can use 1D
		   conservation of momentum equations to calculate 
		   the final velocity along the x-axis. */
		PVector[] vFinal = {  
		    new PVector(), new PVector()
		};

		// final rotated velocity for b[0]
		//vFinal[0].x = ((m - other.m) * vTemp[0].x + 2 * other.m * vTemp[1].x) / (m + other.m);
		vFinal[0].x = vTemp[1].x;  //Assuming m and other.m are equal!
		vFinal[0].y = vTemp[0].y;

		// final rotated velocity for b[0]
		//vFinal[1].x = ((other.m - m) * vTemp[1].x + 2 * m * vTemp[0].x) / (m + other.m);
		vFinal[1].x = vTemp[0].x;
		vFinal[1].y = vTemp[1].y;

		// hack to avoid clumping
		bTemp[0].x += vFinal[0].x;
		bTemp[1].x += vFinal[1].x;

		/* Rotate ball positions and velocities back
		   Reverse signs in trig expressions to rotate 
		   in the opposite direction */
		// rotate balls
		PVector[] bFinal = { 
		    new PVector(), new PVector()
		};

		bFinal[0].x = cosine * bTemp[0].x - sine * bTemp[0].y;
		bFinal[0].y = cosine * bTemp[0].y + sine * bTemp[0].x;
		bFinal[1].x = cosine * bTemp[1].x - sine * bTemp[1].y;
		bFinal[1].y = cosine * bTemp[1].y + sine * bTemp[1].x;

		// update balls to screen position
		//otherPos.x = position.x + bFinal[1].x;
		//otherPos.y = position.y + bFinal[1].y;
		otherPos=PVector.add(position,bFinal[1]);
		position.add(bFinal[0]);

		//px       =position.x;
		//py       =position.y;
		//ofield.px=otherPos.x;
		//ofield.py=otherPos.y;

		int ii=colArray.size();
		int jj=ofield.colArray.size();
		colArray.add(ofield);
		ofield.colArray.add(this);
		forXArray.add(position.x-px);
		forYArray.add(position.y-py);
		ofield.forXArray.add(otherPos.x-ofield.px);
		ofield.forYArray.add(otherPos.y-ofield.py);	
		
                //px+=forXArray.get(ii);
		//py+=forYArray.get(ii);
		//ofield.px+=ofield.forXArray.get(jj);
		//ofield.py+=ofield.forYArray.get(jj);

		// update velocities
		vx       =		velocity.x = cosine * vFinal[0].x - sine * vFinal[0].y;
		vy       =		velocity.y = cosine * vFinal[0].y + sine * vFinal[0].x;
		ofield.vx=		otherVel.x = cosine * vFinal[1].x - sine * vFinal[1].y;
		ofield.vy=		otherVel.y = cosine * vFinal[1].y + sine * vFinal[1].x;

		
                if (verbosity==2){
		    println(colArray.size()       +" Fixed this  f: "+toString()+ " -- "+(dist(px,py,ofield.px, ofield.py)<(fr1 + ofield.fr1)));
		    println(ofield.colArray.size()+" Fixed other f: "+ofield.toString());
		}
	    }

	    
	    
	    
	}//CLOSURE: collisionCheck function
	
    } // CLOSURE:  Class Field



    
    // ####################################################################################################
    //
    // ####################################################################################################    
    class Particle {
	float px=0, py=0; 
	float vx=0, vy=0;
	float ax=0, ay=0;

	World world;

	Particle( World world ) {
	    this.world = world;
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS PARTICLE
	public void update() {                      
	    float tx=px;
	    float ty=py;

	    vx*=world.f1;
	    vy*=world.f1;
	    vx+=ax;
	    vy+=ay;
	    ax*=world.f2;
	    ay*=world.f2;
	    px+=vx;
	    py+=vy;

	    if (world.open) {
		if ( px < 0 ) px+= width;
		if ( py < 0 ) py+= height;
		if (px > width) px-= width;
		if (py > height)  py -=height;
	    } else {
		if (px > width || px < 0) {
		    vx = -vx;
		    ax = -ax;
		    px = tx;
		    py = ty;
		}
		if (py > height || py < 0) {
		    vy = -vy;
		    ay = -ay;
		    px = tx;
		    py = ty;
		}
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS PARTICLE
	public void addForce( float fx, float fy ) {    
	    ax += fx;
	    ay += fy;
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS PARTICLE
	public void draw() {                           
	    float vel = sqrt(sq(vx)+sq(vy))*10;    
	    stroke(255-min(vel, 10)*25.5f, 255, 255);
	    rect( px, py, 0, 0);
	    //point(px,py);  //HERE
	}
    }


    // ####################################################################################################
    //
    // ####################################################################################################
    //CLASS SLIDER
    class Slider {
	float x, y, w, h;
	float value;
	float min, max;
	String name;

	boolean active = false;

	Slider( String name, float x, float y, float w, float h, float min, float max, float value ) {
	    this.name = name;
	    this.x = x;
	    this.y = y;
	    this.w = w;
	    this.h = h;
	    this.min = min;
	    this.max = max;
	    this.value = value;
	}

	//CLASS SLIDER
	public void draw() {
	    fill(200);
	    text(name, x, y+h-3 );
	    stroke(50);
	    fill(50);
	    rect(x+50, y, w, h);
	    noStroke();
	    fill(active?200:128);
	    rect(x+50, y, (value-min)/(max-min)*w, h+1);
	    fill(250);
	    text(""+value, x+60, y+h-3 );
	}

	//CLASS SLIDER
	public boolean inside(float tx, float ty) {
	    return (tx >= 50+x && tx <= x+50+w && ty>=y && ty <= y+h);
	}

	//CLASS SLIDER
	public void slide(float px, float mx) {
	    value+=(mx-px)*(max-min)/w;//+min;
	    if ( value < min ) value = min;
	    if ( value > max ) value = max;
	}
    }


    // ####################################################################################################
    //
    // ####################################################################################################  ??.
    class World {
	// damping factor for velocity
	float f1 = .98f;
	// damping factor for forces
	float f2 = .3f;

	float r1 = 4;
	float r2 = 30;
	float r3 = 60;

	float pfs = 0.05f;
	float pfa = 0.005f;
	float pfc = .4f;  

	// damping factor for velocity
	float ff1 = .80f;
	// damping factor for forces
	float ff2 = .3f;

	float ffs = 0.1f;
	float fchase = 0.0; //0.01f;
	float master_fr1 = 30;//80;
	float master_fr2 = 0;//100;
	float master_fr3 = 5;//40;

	float parNThresh=10;              //MUTABLE
	//float densityThresh=0.001;      //MUTABLE
	boolean invertThreshEffect=false; //MUTABLE
	float minFR1=0.5;
	float fieldSpeed=0.5;  

	boolean open=false;

	private Particle[] particles;
	private Field[] fields;

	World( int fieldcount, int particlecount) {
	    initWorld(fieldcount, particlecount);
	}

	

	// ####################################################################################################
        //
        // ####################################################################################################  ??.
	//CLASS WORLD
	public void initWorld(int fieldcount, int particlecount) {
	    randomSeed( seed );
	    surface.setSize(wWidth,wHeight);

	    //density field value: parNThresh
	    //inversion flag:    invertThreshEffect
	    densityFlag=false;
	    verbosity=0;
	    showFieldLines=true;

	    //updateAllFields();         //CHANGED: particle_fields4    
	    updateRunningStatus(true);
     
	    fields=null;
	    particles=null;
	    
            //fieldcount=width/(2*master_fr1)*height/(2*master_fr1);
	    
	    fields = new Field[fieldcount];
	    particles = new Particle[particlecount] ;

	    for ( int i=0; i<fields.length; i++) {
		fields[i] = new Field();		
		fields[i].updateAllFields(master_fr1,master_fr2,master_fr3,minFR1);
		fields[i].vx=random(-fieldSpeed,fieldSpeed);
	        fields[i].vy=random(-fieldSpeed,fieldSpeed);

		float tx=0;
		float ty=0;
		float tr=2.0*master_fr1;
		boolean found=true;
		int nTries=0;

		while(found==true){
		    tx=random(master_fr1,width-master_fr1);//random(width*.8f+50);
		    ty=random(master_fr1,height-master_fr1);//random(height*.8f+50);

		    //tx=(int((i)%(width/tr))+1) * tr;
                    //ty=(int((i)/(width/tr))+1) * tr;

		
		    found=false;
		    for(int j=0;j<i && !found;j++){
			found = dist(fields[j].px,fields[j].py,tx,ty) < 2.0*master_fr1;
		        if(found){			    
			    tx=random(master_fr1,width-master_fr1);//random(width*.8f+50);
			    ty=random(master_fr1,height-master_fr1);//random(height*.8f+50);			    
			    j=0;
			    
			    if( (nTries++) >10000){
				println("@@@@@ERROR: Forced exit!");
				println("Max number of tries exceeded! Failed to place field " + i + " out of " + fieldcount);
				println("Probably max number of fields allowed with fr1=" + master_fr1 + " is " + (2*master_fr1)*height/(2*master_fr1));
				System.exit(0);
			    }
		        }
		    }
			    
		}
	     	

		//Assigns a non-iverlapping position with existing fields
		//println("Placing "+nf(i,2)+" "+nf(j,2)+" -  "+nf(master_fr1,2)+" "+dist(fields[j].px,fields[j].py,tx,ty));
		fields[i].px = tx;
		fields[i].py = ty;
	    }	     
	
	    //updateAllFields();

	    for ( int i=0; i< particles.length; i++) {
		particles[i] = new Particle(this);
		particles[i].px = random(width);
		particles[i].py = random(height);
	    }	    
	}

	// ####################################################################################################
	//
	// ####################################################################################################
	//CLASS WORLD
	public void initWorld(Slider[] _newsliders, Slider[] _sliders) {

	    for ( int i=0; i< _newsliders.length; i++) {
		changeParamNew( i, _newsliders[i].value);
	    }	  
	    //world = new World( fieldCount, particleCount);  
	    for ( int i=0; i< _sliders.length; i++) {
		changeParam( i, _sliders[i].value);
	    }

	    initWorld(int(_newsliders[2].value), int(_newsliders[1].value) );
	}


	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void changeParam(int idx, float value ) {
	    switch(idx) {
	    case 0: 
		f1 = value; 
		break;
	    case 1: 
		f2 = value; 
		break;
	    case 2: 
		r1 = value; 
		break;
	    case 3: 
		r2 = value; 
		break;
	    case 4: 
		r3 = value; 
		break;
	    case 5: 
		pfs = value; 
		break;
	    case 6: 
		pfa = value; 
		break;
	    case 7: 
		pfc = value; 
		break;
	    case 8: 
		ff1 = value; 
		break;
	    case 9: 
		ff2 = value; 
		break;
	    case 10: 
		master_fr1 = value; 
		break;
	    case 11: 
		master_fr2 = value; 
		break;
	    case 12: 
		master_fr3 = value; 
		break;
	    case 13: 
		ffs = value; 
		break;
	    case 14: 
		fchase = value; 
		break;
	    case 15: 
		 parNThresh = value; 
		break;
	    case 16: 
		 minFR1 = value; 
		break;
	    case 17: 
		 fieldSpeed = value; 
		break;
	    }

	    for ( int i=0; fields!=null && i<fields.length; i++){
		fields[i].updateAllFields(master_fr1,master_fr2,master_fr3,minFR1);
		fields[i].vx=random(-fieldSpeed,fieldSpeed);
	        fields[i].vy=random(-fieldSpeed,fieldSpeed);
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void changeParamNew(int idx, float value ) {
	    switch(idx) {
	    case 0: 
		seed= PApplet.parseInt(value); 
		break;
	    case 1: 
		particleCount = PApplet.parseInt(value); 
		break;
	    case 2: 
		fieldCount = PApplet.parseInt(value); 
		break;
	    case 3: 
		wWidth = PApplet.parseInt(value); 
		break;
	    case 4: 
		wHeight = PApplet.parseInt(value); 
		break;
	    }
	}
	

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void update() {                   
      
	    applyDensityCorrections();

	    for ( int j=0; j<fields.length; j++) {
		//float _ax=0,_ay=0;
		//int _ctr=0; 
		for ( int i=0; i< particles.length; i++) {      


		    float lx = particles[i].px - fields[j].px;
		    float ly = particles[i].py - fields[j].py;

		    if (open) {
			if ( abs(lx) > abs(lx+width)) lx+=width;
			if ( abs(ly) > abs(ly+height)) lx+=height;
		    }

		    float l = wdist(particles[i].px, particles[i].py, fields[j].px, fields[j].py);
		    if ( l< fields[j].fr3 && l> 1) {         
			//float f = 10.0/sq(l);
			float f = 1.0f/l;
			lx = lx/l*f;
			ly = ly/l*f;          
			particles[i].addForce(lx, ly);
			// _ax+=lx;
			//_ay+=ly;
			//_ctr++;
		    }
		    //DISABLED:: particles[i].addForce(_ax, _ay);
		}
	    }

	    //COMENT### Fater way to compare between field pairs and using action-reaction law
	    //for (int i=0; i<n-1; i++)
	    //for (int j=i+1; j<n; j++) {
	    //println("FWD pair="+i+" "+j);
	    //println("           BCK pair="+j+" "+i);
	    //}
	    //sep();


	    for ( int i=0; i<fields.length; i++) {
		//sep( fields[i] );
		//sep_tapeWorms( fields[i] );
		//DISABLED::
		attract( fields[i] );
		for ( int j=i+1; j<fields.length; j++) {
		    //DISABLED::
		    fields[i].checkCollision(fields[j]);
		}		
		fields[i].update();		
	    }

	    //-------------------------------------------	    
	     //Reset/init boundary bouncing....
	    for (int i=0; i<fields.length; i++){
		fields[i].speedXflipFlag=false;
		fields[i].speedYflipFlag=false;
	    }

	    int maxiter=10;
	    boolean c = true;
	    for (int cc=0; cc<maxiter & c; cc++) {
		c =  collide_tapeWorms();
	    }

	    //Flip velocity if field bounced from any wall
	    for (int i=0; i<fields.length; i++){
		if(fields[i].speedXflipFlag==true)
		    fields[i].vx*=-1;
		if(fields[i].speedYflipFlag==true)
		    fields[i].vy*=-1;
	    }
            //-------------------------------------------
      
	    for ( int i=0; i< particles.length; i++) {
		//DISABLED::
		swarm(particles[i]);      
		//DISABLED::
		particles[i].update();
	    }

	    //applyDensityCorrections();
	    printInformation();
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void applyDensityCorrections() {            

	    for ( int i=0; i<fields.length; i++) {
		Field f=fields[i];
		//float cpd=f.pc/float(particleCount);     //Current particle density
		float cpn=f.pc;                            //Current particle number

		boolean effectFlag=cpn>parNThresh;

		if (invertThreshEffect==true)
		    effectFlag=!effectFlag;
		
                f.dfr1Flag=false;
		if (effectFlag) {
		    if (f.fr1>f.min_fr1) {f.fr1-=f.deltafr1;f.dfr1Flag=true;}
		    if (f.fr2>f.min_fr2) f.fr2-=f.deltafr2;  
		    if (f.fr3>f.min_fr3) f.fr3-=f.deltafr3;
		} else {
		    if (f.fr1<f.max_fr1) {f.fr1+=f.deltafr1;f.dfr1Flag=true;}
		    if (f.fr2<f.max_fr2) f.fr2+=f.deltafr2;
		    if (f.fr3<f.max_fr3) f.fr3+=f.deltafr3;
		}
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	boolean collide_tapeWorms() {  

	    boolean c = false;   

	    // bubble collisions
	    for (int i=0; i<fields.length; i++) {
		for (int j=0; j<fields.length; j++) {
		    c |= fields[i].collide_tapeWorms(fields[j]);
		}	    

		// wall collisions
		if (fields[i].px<fields[i].fr1) { 
		    fields[i].px=fields[i].fr1 ;
		    fields[i].speedXflipFlag=true;
		    c = true;
		}
		if (fields[i].px>width-fields[i].fr1) { 
		    fields[i].px = width-fields[i].fr1;
		    fields[i].speedXflipFlag=true;
		    c = true;
		}
		if (fields[i].py<fields[i].fr1) { 
		    fields[i].py = fields[i].fr1;
		    fields[i].speedYflipFlag=true;
		    c = true;
		}
		if (fields[i].py>height-fields[i].fr1) { 
		    fields[i].py = height-fields[i].fr1;
		    fields[i].speedYflipFlag=true;
		    c = true;
		}
	    }

	    return c;
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	void printInformation() {
	    if (verbosity==1 && frameCount%6==0) {
		String msg="###########";
		msg += "\n Open field=" + open + " parNThresh="+parNThresh + " InvFlag="+invertThreshEffect;
		msg += "\n parCount=" + particleCount + " fieldCount="+fieldCount;
		msg += "\n f1 f2=" + f1 + " " + f2;
		msg += "\n r1/2/3=" + r1+ " "+r2+ " "+r3;
		msg += "\n pfs pfc=" + pfs + " " + pfc;
		msg += "\n ff1 ff2=" + ff1 + " " + ff2;
		msg += "\n fr1/2/3=" + nf(master_fr1, 3, 3)+ " "+nf(master_fr2, 3, 3)+ " "+nf(master_fr3, 3, 3);
		msg += "\n ffs fchase=" + ffs + " " + fchase;
		msg += "\n";
		for ( int i=0; i<fields.length; i++) {
		    msg += "\n ID:"+nf(i, 2)+"=>\t" + nf(fields[i].fr1, 3, 3)+ "\t"+nf(fields[i].fr2, 3, 3)+ "\t"+nf(fields[i].fr3, 3, 3) + "\t" + nf(fields[i].pc, 3, 0);
		}


		println(msg);
	    }
	}


	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void draw() {                    

	    //COMMENT###
	    //Only if mouse position changes
	    //if (densityFlag==true && mouseX!=pmouseX) {
	    //	parNThresh=map(mouseX, 0, width, 5, 150);
	    //	println("Field threshold updated to "+parNThresh);
	    //}

	    for ( int i=0; i<fields.length; i++) {
		fields[i].draw();
	    }
	    for ( int i=0; i< particles.length; i++) {//COMMENT###
		particles[i].draw();
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void swarm(Particle p) {               
	    float tsx=0, tsy=0;        
	    float tax=0, tay=0;
	    float tcx=0, tcy=0;

	    int counts = 0;
	    int counta = 0;
	    int countc = 0;

	    
	    for (int i=0; i<world.particles.length; i++) {
		Particle other = particles[i];
		if ( p != other) {

		    float d = wdist(p.px, p.py, other.px, other.py);

		    if ( d < r1) {
			counts++;

			tsx+=other.px;
			tsy+=other.py;
		    } else if (d < r3) {        
			countc++;            
			tcx+=other.px;
			tcy+=other.py;
		    }  
		    if ( d < r2) {
			counta++;
			tax+=other.vx;
			tay+=other.vy;
		    }
		}
	    }
	    if ( counts > 0 ) { 
		tsx/=counts;
		tsy/=counts;
		tsx = p.px - tsx;
		tsy = p.py - tsy;

		float l = wdist(0, 0, tsx, tsy);
		if ( l > 0) {
		    tsx/=l;
		    tsy/=l;
		}
		tsx*=pfs;
		tsy*=pfs;
		p.addForce(tsx, tsy);
	    }

	    if ( counta > 0 ) {
		tax = tax/counta;
		tay = tay/counta;

		float l = wdist(tax, tay, 0, 0);
		if (l > 0) {
		    tax=tax/l;
		    tay=tay/l;
		}
		tax*=pfa;
		tay*=pfa;
		p.addForce(tax, tay);
	    }

	    if ( countc> 0 ) {
		tcx/=countc;
		tcy/=countc;
		tcx = p.px - tcx;
		tcy = p.py - tcy;

		float l = wdist(0, 0, tcx, tcy);
		if (l > 0) {
		    tcx/=l;
		    tcy/=l;
		}
		tcx*=(pfc-0.5f)/1000.0f;
		tcy*=(pfc-0.5f)/1000.0f;
		p.addForce(tcx, tcy);
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################  
	public void sep() {                       //CLASS WORLD
	    for (int i=0; i<fields.length-1; i++) {
		Field f=fields[i];
		for (int j=i+1; j<fields.length; j++) {
		    float tmpx =0, tmpy =0;
		    //int count = 0;
		    Field other=fields[j];
		    float d = wdist(f.px, f.py, other.px, other.py);
		    tmpx=f.px - other.px;
		    tmpy=f.py - other.py;
		    float l = dist(0, 0, tmpx, tmpy);  //COMMENT### dist() for wdit()
		    if (l!=0) {
			tmpx/=l;
			tmpy/=l;
		    }
		    if ( d < f.fr1) {		 
			f.addForce(tmpx*ffs, tmpy*ffs);
		    }
		    if ( d < other.fr1) {
			other.addForce(-tmpx*ffs, -tmpy*ffs);  //INVERT signs
		    }
		}
	    }
	}

        // ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void sep(Field f) {                  
	    float tmpx =0, tmpy =0;
	    int count = 0;

	    for (int i=0; i<fields.length; i++) {
		Field other = fields[i];
		if ( f != other) {
		    float d = wdist(f.px, f.py, other.px, other.py); 
		    if ( d < master_fr1) {
			count++;
			tmpx+=other.px;
			tmpy+=other.py;
		    }
		}
	    }

	    if ( count > 0 ) {      
		tmpx/=count;
		tmpy/=count;
		tmpx = f.px - tmpx;
		tmpy = f.py - tmpy;

		float l = wdist(0, 0, tmpx, tmpy);
		tmpx/=l;
		tmpy/=l;
		tmpx*=ffs;
		tmpy*=ffs;
		//DISABLED::
		f.addForce(tmpx, tmpy);
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD        
	public void sep_tapeWorms(Field f) {        
	    float speed=0.5;

	    for (int i=0; i<fields.length; i++) {
		Field other = fields[i];
		if ( f != other) {
		    float d = wdist(f.px, f.py, other.px, other.py);
		    speed=f.fr1+other.fr1 - d;
		    //if( d < (f.fr1+other.fr1)){
		    if (speed > 0) {
			if (verbosity==2) println("Speed="+speed+" ax="+f.ax);
			//speed=0.80;
			float dir = atan2(f.py-other.py, f.px-other.px);
			//f.px += speed * cos(dir);
			//f.py += speed * sin(dir);
			f.addForce(speed * cos(dir), speed * sin(dir));	    
			//other.px -= speed * cos(dir);
			//other.py -= speed * sin(dir);
			//!!!!!!!other.addForce(-speed * cos(dir),-speed * sin(dir));

			if (verbosity==2) println("           ax="+f.ax);

			//if(f.px<0) f.px=0;	   
			//if(f.py<0) f.py=0;	    
			//if(f.px>width) f.px=width;	    
			//if(f.py>height) f.py=height;

			//if(other.px<0) other.px=0;	   
			//if(other.py<0) other.py=0;	    
			//if(other.px>width) other.px=width;	    
			//if(other.py>height) other.py=height;
		    }
		}
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public void attract(Field f) {                
	    float tmpx =0, tmpy =0;
	    int count = 0;
	    f.pc=0;
	    for (int i=0; i<particles.length; i++) {
		Particle other = particles[i];

		float d = wdist( f.px, f.py, other.px, other.py); 
		if ( d < f.fr2) {
		    count++;
		    tmpx+=other.px;
		    tmpy+=other.py;
		}

		if ( d < f.fr1 )  //COMMENT###
		    f.pc++;
	    }


	    if ( count > 0 && fchase>0.001 ) {    
		tmpx/=count;
		tmpy/=count;
		tmpx = tmpx - f.px;
		tmpy = tmpy - f.py;
		float l = wdist(0, 0, tmpx, tmpy);
		tmpx/=l;
		tmpy/=l;
		tmpx*=fchase;
		tmpy*=fchase;

		//DISABLED::
		f.addForce(tmpx, tmpy);
	    }
	}

	// ####################################################################################################
        //
        // ####################################################################################################
	//CLASS WORLD
	public float wdist( float x1, float y1, float x2, float y2 ) {
	    if ( !open ) {
		return dist( x1, y1, x2, y2);
	    } else { 
		float dx = min( abs(x1-x2), abs(x1+width-x2));
		float dy = min( abs(y1-y2), abs(y1+height-y2));
		//COMENT### Checking min independently of x and y is not sufficient.
		//If distance is closer along one dimension when warping around,
		//it could not be close in the other. It needs to be checked!
		return sqrt( sq(dx) + sq(dy));
	    }
	}
    }

    // ####################################################################################################
    //
    // ####################################################################################################  
    static public void main(String[] passedArgs) {
	String[] appletArgs = new String[] { "particle_fields4" };
	if (passedArgs != null) {
	    PApplet.main(concat(appletArgs, passedArgs));
	} else {
	    PApplet.main(appletArgs);
	}
    }
}